package net.swengineer.codegraph.finders;

import com.intellij.psi.*;
import com.intellij.psi.search.PsiSearchScopeUtil;
import com.intellij.psi.search.SearchScope;
import com.intellij.psi.search.searches.ClassInheritorsSearch;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Finds usages of a PsiClass in a given scope
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class ClassUsageFinder {
    private PsiClass psiClass;
    private SearchScope searchScope;

    public static ClassUsageFinder create(PsiClass psiClass, SearchScope searchScope) {
        return new ClassUsageFinder(psiClass, searchScope);
    }

    private ClassUsageFinder(PsiClass psiClass, SearchScope searchScope) {
        this.psiClass = psiClass;
        this.searchScope = searchScope;
    }

    public Set<Usage> getAllUsages() {
        Set<Usage> usages = new HashSet<Usage>();

        usages.addAll(getContainedMethodsUsages());
        usages.addAll(getContainedFieldsUsages());
        usages.addAll(getSuperClassesUsages());
        usages.addAll(getInterfacesUsages());
        usages.addAll(getExtendingOrImplementingClassesUsages());
        usages.addAll(getInnerClassesUsages());
        usages.addAll(getContainingClassUsage());

        return usages;
    }

    public Set<Usage> getContainedMethodsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiMethod[] methods = psiClass.getMethods();
        for (PsiMethod psiMethod : methods) {
            Usage containedMethod = new Usage(psiClass, psiMethod, Usage.Type.CONTAINS);
            usages.add(containedMethod);
        }
        return usages;
    }

    public Set<Usage> getContainedFieldsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiField[] fields = psiClass.getFields();
        for (PsiField psiField : fields) {
            Usage containedMethod = new Usage(psiClass, psiField, Usage.Type.CONTAINS);
            usages.add(containedMethod);
        }
        return usages;
    }

    public Set<Usage> getSuperClassesUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiReferenceList extendsList = psiClass.getExtendsList();
        addReferenceListUsages(usages, psiClass, extendsList, Usage.Type.EXTENDS, searchScope);
        return usages;
    }

    public Set<Usage> getInterfacesUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiReferenceList implementsList = psiClass.getImplementsList();
        addReferenceListUsages(usages, psiClass, implementsList, Usage.Type.IMPLEMENTS, searchScope);
        return usages;
    }

    private void addReferenceListUsages(Set<Usage> usages, PsiClass psiClass, PsiReferenceList extendingClasses, Usage.Type usageType, SearchScope searchScope) {
        PsiJavaCodeReferenceElement[] references = new PsiJavaCodeReferenceElement[0];
        if (extendingClasses != null) {
            references = extendingClasses.getReferenceElements();
        }
        for (PsiJavaCodeReferenceElement reference : references) {
            PsiClass extendingClass = (PsiClass) reference.resolve();
            if (PsiSearchScopeUtil.isInScope(searchScope, extendingClass)) {
                Usage usage = new Usage(psiClass, extendingClass, usageType);
                usages.add(usage);
            }
        }
    }

    public Set<Usage> getExtendingOrImplementingClassesUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        Collection<PsiClass> extendingClasses = ClassInheritorsSearch.search(psiClass, false).findAll();
        for (PsiClass extendingClass : extendingClasses) {
            if (!(extendingClass instanceof PsiAnonymousClass)) {
                if (psiClass.isInterface() && !extendingClass.isInterface()) {
                    usages.add(new Usage(extendingClass, psiClass, Usage.Type.IMPLEMENTS));
                } else {
                    usages.add(new Usage(extendingClass, psiClass, Usage.Type.EXTENDS));
                }
            }
        }
        return usages;
    }

    public Set<Usage> getInnerClassesUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiClass[] innerClasses = psiClass.getInnerClasses();
        for (PsiClass innerClass : innerClasses) {
            usages.add(new Usage(psiClass, innerClass, Usage.Type.CONTAINS));
        }
        return usages;
    }

    public Set<Usage> getContainingClassUsage() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiClass containingClass = psiClass.getContainingClass();
        if (containingClass != null) {
            usages.add(new Usage(containingClass, psiClass, Usage.Type.CONTAINS));
        }
        return usages;
    }
}
