package net.swengineer.codegraph.finders;

import com.intellij.psi.*;
import com.intellij.psi.search.PsiSearchScopeUtil;
import com.intellij.psi.search.SearchScope;
import com.intellij.psi.search.searches.ReferencesSearch;
import com.intellij.psi.util.PsiTreeUtil;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Finds usages of a PsiField in a given scope
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class FieldUsageFinder {
    private PsiField psiField;
    private SearchScope searchScope;

    public static FieldUsageFinder create(PsiField psiField, SearchScope searchScope) {
        return new FieldUsageFinder(psiField, searchScope);
    }

    private FieldUsageFinder(PsiField psiField, SearchScope searchScope) {
        this.psiField = psiField;
        this.searchScope = searchScope;
    }

    public Set<Usage> getAllUsages() {
        Set<Usage> usages = new HashSet<Usage>();

        usages.addAll(getContainingClassUsage());
        usages.addAll(getFieldTypeUsage());
        usages.addAll(getUsingMethodsUsages());

        return usages;
    }

    public Set<Usage> getContainingClassUsage() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiClass containingClass = psiField.getContainingClass();
        usages.add(new Usage(containingClass, psiField, Usage.Type.CONTAINS));
        return usages;
    }

    public Set<Usage> getFieldTypeUsage() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiTypeElement typeElement = psiField.getTypeElement();
        PsiJavaCodeReferenceElement componentReferenceElement;
        if (typeElement != null) {
            componentReferenceElement = typeElement.getInnermostComponentReferenceElement();
            if (componentReferenceElement != null) {
                PsiElement element = componentReferenceElement.resolve();
                if (element instanceof PsiClass) {
                    if (PsiSearchScopeUtil.isInScope(searchScope, element)) {
                        usages.add(new Usage(psiField, (PsiClass) element, Usage.Type.TYPEOF));
                    }
                }
            }

        }

        return usages;
    }

    public Set<Usage> getUsingMethodsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        Collection<PsiReference> references = ReferencesSearch.search(psiField, searchScope).findAll();
        for (PsiReference reference : references) {
            if (reference instanceof PsiReferenceExpression) {
                PsiReferenceExpression refExpr = (PsiReferenceExpression) reference;
                PsiMethod methodParent = PsiTreeUtil.getParentOfType(refExpr, PsiMethod.class);
                if (methodParent != null && !methodParent.isConstructor()) {
                    PsiElement parent = refExpr.getParent();
                    if (parent instanceof PsiAssignmentExpression) {
                        PsiAssignmentExpression assignmentExpr = (PsiAssignmentExpression) parent;
                        if (refExpr.equals(assignmentExpr.getLExpression())) {
                            usages.add(new Usage(methodParent, psiField, Usage.Type.ASSIGNS));
                        } else {
                            usages.add(new Usage(methodParent, psiField, Usage.Type.READS));
                        }
                    } else {
                        usages.add(new Usage(methodParent, psiField, Usage.Type.READS));
                    }
                }
            }
        }
        return usages;
    }
}

