package net.swengineer.codegraph.finders;

import com.intellij.psi.*;
import com.intellij.psi.search.PsiSearchScopeUtil;
import com.intellij.psi.search.SearchScope;
import com.intellij.psi.search.searches.OverridingMethodsSearch;
import com.intellij.psi.search.searches.ReferencesSearch;
import com.intellij.psi.util.PsiTreeUtil;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Finds usages of a PsiMethod in a given scope
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class MethodUsageFinder {
    private PsiMethod psiMethod;
    private SearchScope searchScope;

    public static MethodUsageFinder create(PsiMethod psiMethod, SearchScope searchScope) {
        return new MethodUsageFinder(psiMethod, searchScope);
    }

    private MethodUsageFinder(PsiMethod psiMethod, SearchScope searchScope) {
        this.psiMethod = psiMethod;
        this.searchScope = searchScope;
    }

    public Set<Usage> getAllUsages() {
        Set<Usage> usages = new HashSet<Usage>();

        usages.addAll(getCallingMethodsUsages());
        usages.addAll(getOverridingMethodsUsages());
        usages.addAll(getCalledMethodUsages());
        usages.addAll(getContainingClassUsage());
        usages.addAll(getSuperMethodsUsages());
        usages.addAll(getUsedFieldsUsages());
        usages.addAll(getContainedAnonymousClassesUsages());

        return usages;
    }

    public Set<Usage> getContainedAnonymousClassesUsages() {
        final Set<Usage> usages = new HashSet<Usage>();
        psiMethod.accept(new PsiRecursiveElementVisitor() {
            @Override
            public void visitClass(PsiClass psiClass) {
                usages.add(new Usage(psiMethod, psiClass, Usage.Type.CONTAINS));
            }
        });
        return usages;
    }

    public Set<Usage> getCallingMethodsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        Collection<PsiReference> references = ReferencesSearch.search(psiMethod, searchScope).findAll();
        for (PsiReference reference : references) {
            PsiElement context = reference.getElement().getContext();
            if (context instanceof PsiMethodCallExpression) {
                PsiMethod callingMethod = PsiTreeUtil.getParentOfType(context, PsiMethod.class);
                usages.add(new Usage(callingMethod, psiMethod, Usage.Type.CALLS));
            }
        }
        return usages;
    }

    public Set<Usage> getOverridingMethodsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        Collection<PsiMethod> methods = OverridingMethodsSearch.search(psiMethod, searchScope, false).findAll();
        for (PsiMethod method : methods) {
            Usage.Type type;
            if (psiMethod.getContainingClass().isInterface()) {
                type = Usage.Type.IMPLEMENTS;
            } else {
                type = Usage.Type.OVERRIDES;
            }
            usages.add(new Usage(method, psiMethod, type));
        }
        return usages;
    }

    public Set<Usage> getCalledMethodUsages() {
        final Set<Usage> usages = new HashSet<Usage>();
        PsiRecursiveElementVisitor visitor = new PsiRecursiveElementVisitor() {
            @Override
            public void visitMethodCallExpression(PsiMethodCallExpression expression) {
                PsiMethod targetMethod = expression.resolveMethod();
                if (PsiSearchScopeUtil.isInScope(searchScope, targetMethod)) {
                    usages.add(new Usage(psiMethod, targetMethod, Usage.Type.CALLS));
                }
                super.visitMethodCallExpression(expression);
            }
        };
        psiMethod.accept(visitor);
        return usages;
    }

    public Set<Usage> getContainingClassUsage() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiClass psiClass = psiMethod.getContainingClass();
        usages.add(new Usage(psiClass, psiMethod, Usage.Type.CONTAINS));
        return usages;
    }

    public Set<Usage> getSuperMethodsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        PsiMethod[] superMethods = psiMethod.findSuperMethods();
        for (PsiMethod superMethod : superMethods) {
            PsiClass methodClass = superMethod.getContainingClass();
            if (PsiSearchScopeUtil.isInScope(searchScope, superMethod)) {
                if (methodClass.isInterface()) {
                    usages.add(new Usage(psiMethod, superMethod, Usage.Type.IMPLEMENTS));
                } else {
                    usages.add(new Usage(psiMethod, superMethod, Usage.Type.OVERRIDES));
                }
            }
        }
        return usages;
    }

    public Set<Usage> getUsedFieldsUsages() {
        Set<Usage> usages = new HashSet<Usage>();
        UsedFieldsFinder usedFieldsFinder = new UsedFieldsFinder(psiMethod, usages, searchScope);
        usedFieldsFinder.visitElement(psiMethod);
        return usages;
    }

    private static class UsedFieldsFinder extends PsiRecursiveElementVisitor {
        PsiMethod sourceMethod;
        SearchScope searchScope;
        Set<Usage> fieldUsages;

        private UsedFieldsFinder(PsiMethod sourceMethod, Set<Usage> fieldUsages, SearchScope searchScope) {
            this.sourceMethod = sourceMethod;
            this.fieldUsages = fieldUsages;
            this.searchScope = searchScope;
        }

        @Override
        public void visitReferenceExpression(PsiReferenceExpression expression) {
            PsiElement element = expression.resolve();
            if (element instanceof PsiField) {
                PsiField psiField = (PsiField) element;
                PsiElement context = expression.getContext();
                if (context instanceof PsiAssignmentExpression) {
                    PsiAssignmentExpression assignment = (PsiAssignmentExpression) context;
                    PsiExpression leftExpr = assignment.getLExpression();
                    if (leftExpr instanceof PsiReferenceExpression) {
                        PsiElement assignedField = ((PsiReferenceExpression) leftExpr).resolve();
                        if (psiField.equals(assignedField)) {
                            fieldUsages.add(new Usage(sourceMethod, psiField, Usage.Type.ASSIGNS));
                        } else {
                            fieldUsages.add(new Usage(sourceMethod, psiField, Usage.Type.READS));
                        }
                    }
                } else {
                    fieldUsages.add(new Usage(sourceMethod, psiField, Usage.Type.READS));
                }
            }
        }
    }
}
