package net.swengineer.codegraph.graph;

import com.intellij.openapi.Disposable;
import com.intellij.openapi.graph.GraphManager;
import com.intellij.openapi.graph.base.Edge;
import com.intellij.openapi.graph.base.Node;
import com.intellij.openapi.graph.builder.GraphBuilder;
import com.intellij.openapi.graph.builder.GraphBuilderFactory;
import com.intellij.openapi.graph.view.Graph2D;
import com.intellij.openapi.graph.view.Graph2DView;
import com.intellij.openapi.project.Project;
import net.swengineer.codegraph.graph.model.CodeGraphDataModel;
import net.swengineer.codegraph.graph.model.CodeGraphEdge;
import net.swengineer.codegraph.graph.model.CodeGraphFilterer;
import net.swengineer.codegraph.graph.model.CodeGraphNode;
import net.swengineer.codegraph.graph.presentation.CodeGraphPresentationModel;
import net.swengineer.codegraph.plugin.util.CodeGraphLogger;

import javax.swing.*;

/**
 * Hold a GraphBuilder which holds data and presentation models
 * and a CodeGraphFilterer which controls filtering
 * this class is used to create a graph, add/remove items, updating
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraph implements Disposable {
    private GraphBuilder<CodeGraphNode, CodeGraphEdge> graphBuilder;
    private CodeGraphFilterer graphFilterer;

    public static CodeGraph createGraph(Project project) {
        Graph2D graph = GraphManager.getGraphManager().createGraph2D();
        Graph2DView view = GraphManager.getGraphManager().createGraph2DView();
        CodeGraphDataModel dataModel = new CodeGraphDataModel();
        CodeGraphPresentationModel presentationModel = new CodeGraphPresentationModel(graph);
        return new CodeGraph(project, graph, view, dataModel, presentationModel);
    }

    private CodeGraph(Project project, Graph2D graph, Graph2DView view, CodeGraphDataModel dataModel, CodeGraphPresentationModel presentationModel) {
        graphBuilder = GraphBuilderFactory.getInstance(project).createGraphBuilder(graph, view, dataModel, presentationModel);
        presentationModel.setCodeGraph(this);
    }

    public Project getProject() {
        return graphBuilder.getProject();
    }

    public Graph2D getGraph() {
        return graphBuilder.getGraph();
    }

    public Graph2DView getView() {
        return graphBuilder.getView();
    }

    public CodeGraphDataModel getDataModel() {
        return (CodeGraphDataModel) graphBuilder.getGraphDataModel();
    }

    public CodeGraphPresentationModel getPresentationModel() {
        return (CodeGraphPresentationModel) graphBuilder.getGraphPresentationModel();
    }

    public CodeGraphFilterer getGraphFilterer() {
        return graphFilterer;
    }

    public void initialize() {
        long start = System.currentTimeMillis();
        graphBuilder.initialize();
        graphFilterer = new CodeGraphFilterer(this);
        graphBuilder.addCustomUpdater(graphFilterer);
        long time = System.currentTimeMillis() - start;
        CodeGraphLogger.debug("Graph initalized, took " + time + "ms");
    }

    public JComponent getJComponent() {
        return getView().getJComponent();
    }

    public void clear() {
        getGraph().clear();
    }

    public CodeGraphNode getCodeGraphNode(Node n) {
        return graphBuilder.getNodeObject(n);
    }

    public CodeGraphEdge getCodeGraphEdge(Edge e) {
        return graphBuilder.getEdgeObject(e);
    }

    public Node getNode(CodeGraphNode node) {
        return graphBuilder.getNode(node);
    }

    public Edge getEdge(CodeGraphEdge edge) {
        return graphBuilder.getEdge(edge);
    }

    public void removeNode(CodeGraphNode codeGraphNode) {
        Node node = graphBuilder.getNode(codeGraphNode);
        getDataModel().getNodes().remove(codeGraphNode);
        getGraph().removeNode(node);
    }

    public void removeNode(Node node) {
        getGraph().removeNode(node);
    }

    public GraphBuilder<CodeGraphNode, CodeGraphEdge> getGraphBuilder() {
        return graphBuilder;
    }

    public void updateGraph() {
        long start = System.currentTimeMillis();
        graphBuilder.updateGraph();
        long time = System.currentTimeMillis() - start;
        int modelNumOfNodes = getDataModel().getNodes().size();
        int modelNumOfEdges = getDataModel().getEdges().size();
        CodeGraphLogger.debug("Graph updated (DataModel: " + modelNumOfNodes + " nodes, " + modelNumOfEdges + " edges) , took " + time + "ms");
    }

    public void updateGraphView() {
        long start = System.currentTimeMillis();
        graphBuilder.initialize();
        long time = System.currentTimeMillis() - start;
        CodeGraphLogger.debug("Graph view updated, took " + time + "ms");
    }


    public void fitContent() {
        getView().fitContent();
    }

    public void dispose() {

    }

    public void queueGraphUpdate() {
        graphBuilder.queueUpdate();
    }
}
