package net.swengineer.codegraph.graph.model;

import com.intellij.openapi.graph.builder.GraphDataModel;
import net.swengineer.codegraph.graph.presentation.CodeGraphTypeNames;
import net.swengineer.codegraph.plugin.util.CodeGraphLogger;
import org.jetbrains.annotations.NotNull;

import java.util.*;

/**
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphDataModel extends GraphDataModel<CodeGraphNode, CodeGraphEdge> {
    private Set<CodeGraphNode> nodes;
    private Map<CodeGraphNode, Set<CodeGraphEdge>> nodesEdges;
    private Set<CodeGraphEdge> edges;

    public CodeGraphDataModel() {
        nodes = new HashSet<CodeGraphNode>();
        nodesEdges = new HashMap<CodeGraphNode, Set<CodeGraphEdge>>();
        edges = new HashSet<CodeGraphEdge>();
    }

    public CodeGraphDataModel(Set<CodeGraphNode> nodes, Set<CodeGraphEdge> edges) {
        this.nodes = nodes;
        this.edges = edges;
    }

    @NotNull
    public Collection<CodeGraphNode> getNodes() {
        return nodes;
    }

    @NotNull
    public Collection<CodeGraphEdge> getEdges() {
        return edges;
    }

    @NotNull
    public CodeGraphNode getSourceNode(CodeGraphEdge codeGraphEdge) {
        return codeGraphEdge.getSource();
    }

    @NotNull
    public CodeGraphNode getTargetNode(CodeGraphEdge codeGraphEdge) {
        return codeGraphEdge.getTarget();
    }

    @NotNull
    public String getNodeName(CodeGraphNode codeGraphNode) {
        return codeGraphNode.getName();
    }

    @NotNull
    public String getEdgeName(CodeGraphEdge codeGraphEdge) {
        return CodeGraphTypeNames.getTypeName(codeGraphEdge.getType());
    }

    public CodeGraphEdge createEdge(@NotNull CodeGraphNode from, @NotNull CodeGraphNode to) {
        //will not support edit mode
        return null;
    }

    public void dispose() {
        nodes.clear();
        edges.clear();
        nodes = null;
        edges = null;
    }

    public void addNode(CodeGraphNode node) {
        boolean added = nodes.add(node);
        if (added) {
            CodeGraphLogger.debug("addNode - Added node : " + node);
        } else {
            CodeGraphLogger.debug("addNode - Node already in model : " + node);
        }
    }

    public void addEdge(CodeGraphEdge edge) {
        boolean added = edges.add(edge);
        if (added) {
            CodeGraphLogger.debug("addEdge - Added edge : " + edge);
        } else {
            CodeGraphLogger.debug("addEdge - Edge already in model : " + edge);
        }
        addNode(edge.getSource());
        addNode(edge.getTarget());
        addNodeEdge(edge.getSource(), edge);
        addNodeEdge(edge.getTarget(), edge);
    }

    private void addNodeEdge(CodeGraphNode node, CodeGraphEdge edge) {
        Set<CodeGraphEdge> nodeEdges = nodesEdges.get(node);
        if (nodeEdges == null) {
            nodeEdges = new HashSet<CodeGraphEdge>();
            nodesEdges.put(node, nodeEdges);
        }
        nodeEdges.add(edge);
    }

    public void addAll(Set<CodeGraphEdge> edges) {
        for (CodeGraphEdge edge : edges) {
            addEdge(edge);
        }
    }

    public boolean removeNode(CodeGraphNode node) {
        boolean removed = nodes.remove(node);
        if (removed) {
            CodeGraphLogger.debug("removeNode - Removed node : " + node);
            removeNodeEdges(node);
            nodesEdges.remove(node);
        } else {
            CodeGraphLogger.debug("addNode - Node not in model : " + node);
        }
        return removed;
    }

    private void removeNodeEdges(CodeGraphNode node) {
        Set<CodeGraphEdge> nodeEdges = nodesEdges.get(node);
        if (nodeEdges != null) {
            for (CodeGraphEdge nodeEdge : nodeEdges) {
                removeEdge(nodeEdge);
            }
        }
    }

    public boolean removeEdge(CodeGraphEdge edge) {
        boolean removed = edges.remove(edge);
        if (removed) {
            CodeGraphLogger.debug("removeEdge - Removed edge : " + edge);
        } else {
            CodeGraphLogger.debug("removeEdge - Edge not in model : " + edge);
        }
        return removed;
    }
}
