package net.swengineer.codegraph.graph.model;

import com.intellij.openapi.graph.GraphManager;
import com.intellij.openapi.graph.base.Edge;
import com.intellij.openapi.graph.base.Node;
import com.intellij.openapi.graph.builder.CustomGraphUpdater;
import com.intellij.openapi.graph.util.GraphHider;
import com.intellij.openapi.graph.view.Graph2D;
import com.intellij.openapi.graph.view.Graph2DView;
import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.plugin.util.CodeGraphLogger;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * This Filterer is a CustomGraphUpdater
 * it uses a GraphHider to hide filtered items from the graph
 * the filterer is added as a CustomGraphUpdater to the GraphBuilder
 * and invoked at each update.
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphFilterer extends CustomGraphUpdater {
    private GraphHider graphHider;
    private CodeGraph codeGraph;
    private Map<Usage.Type, Boolean> edgeShouldHideMap;
    private Map<CodeGraphNode.Type, Boolean> nodeShouldHideMap;

    public CodeGraphFilterer(CodeGraph codeGraph) {
        this.codeGraph = codeGraph;
        this.edgeShouldHideMap = new HashMap<Usage.Type, Boolean>();
        this.nodeShouldHideMap = new HashMap<CodeGraphNode.Type, Boolean>();
    }

    public void unfilterAll() {
        graphHider.unhideAll();
    }

    public void filter(CodeGraphNode node) {
        Node n = codeGraph.getNode(node);
        if (n != null) {
            filter(n);
        }
    }

    public void filter(Node node) {
        graphHider.hide(node);
    }

    public boolean shouldFilter(CodeGraphNode node) {
        CodeGraphNode.Type type = node.getType();
        return shouldFilter(type);
    }

    public boolean shouldFilter(CodeGraphNode.Type type) {
        Boolean shouldHide = nodeShouldHideMap.get(type);
        return shouldHide != null && shouldHide;
    }

    public void setShouldFilter(Usage.Type edgeType, boolean value) {
        edgeShouldHideMap.put(edgeType, value);
    }

    public void filter(CodeGraphEdge edge) {
        Edge e = codeGraph.getEdge(edge);
        if (e != null) {
            graphHider.hide(e);
        }
    }

    public boolean shouldFilter(CodeGraphEdge edge) {
        Usage.Type type = edge.getType();
        return shouldFilter(type);
    }

    public boolean shouldFilter(Usage.Type type) {
        Boolean shouldHide = edgeShouldHideMap.get(type);
        return shouldHide != null && shouldHide;
    }

    public void setShouldFilter(CodeGraphNode.Type type, boolean value) {
        nodeShouldHideMap.put(type, value);
    }

    public void update(Graph2D graph, Graph2DView view) {
        long start = System.currentTimeMillis();
        graphHider = GraphManager.getGraphManager().createGraphHider(graph);
        graphHider.setFireGraphEventsEnabled(true);
        graphHider.unhideAll();
        Collection<CodeGraphNode> nodes = codeGraph.getDataModel().getNodes();
        for (CodeGraphNode node : nodes) {
            if (shouldFilter(node)) {
                filter(node);
            }
        }
        Collection<CodeGraphEdge> edges = codeGraph.getDataModel().getEdges();
        for (CodeGraphEdge edge : edges) {
            if (shouldFilter(edge)) {
                filter(edge);
            }
        }
        long time = System.currentTimeMillis() - start;
        CodeGraphLogger.debug("GraphHider update took " + time + "ms");
    }

    public void clear() {
        nodeShouldHideMap.clear();
        edgeShouldHideMap.clear();
    }
}
