package net.swengineer.codegraph.graph.presentation;

import com.intellij.openapi.graph.base.Graph;
import com.intellij.openapi.graph.builder.DeleteProvider;
import com.intellij.openapi.graph.builder.components.BasicGraphPresentationModel;
import com.intellij.openapi.graph.view.*;
import com.intellij.psi.PsiAnonymousClass;
import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiField;
import com.intellij.psi.PsiMethod;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphDataModel;
import net.swengineer.codegraph.graph.model.CodeGraphEdge;
import net.swengineer.codegraph.graph.model.CodeGraphNode;
import net.swengineer.codegraph.graph.presentation.popupmenu.CodeGraphPopupMode;
import org.jetbrains.annotations.NotNull;

/**
 * @author Aviram Segal
 * @version 1.0
 */
public class CodeGraphPresentationModel extends BasicGraphPresentationModel<CodeGraphNode, CodeGraphEdge> {
    private CodeGraph codeGraph;

    public CodeGraphPresentationModel(Graph graph) {
        super(graph);
        setShowEdgeLabels(true);
    }

    public void setCodeGraph(CodeGraph codeGraph) {
        this.codeGraph = codeGraph;
        setGraph(codeGraph.getGraph());
        setGraphBuilder(codeGraph.getGraphBuilder());
    }

    @Override
    public DeleteProvider getDeleteProvider() {
        return new DeleteProvider<CodeGraphNode, CodeGraphEdge>() {
            public boolean canDeleteNode(@NotNull CodeGraphNode node) {
                return true;
            }

            public boolean deleteNode(@NotNull CodeGraphNode node) {
                boolean removed = getDataModel().removeNode(node);
                codeGraph.queueGraphUpdate();
                return removed;
            }

            public boolean canDeleteEdge(@NotNull CodeGraphEdge edge) {
                return true;
            }

            public boolean deleteEdge(@NotNull CodeGraphEdge edge) {
                boolean removed = getDataModel().removeEdge(edge);
                codeGraph.queueGraphUpdate();
                return removed;
            }
        };
    }

    private CodeGraphDataModel getDataModel() {
        return codeGraph.getDataModel();
    }

    @Override
    public void customizeSettings(Graph2DView view, EditMode editMode) {
        PopupMode defaultPopupMode = null;
        ViewMode vm = editMode.getPopupMode();
        if (vm instanceof PopupMode) {
            defaultPopupMode = (PopupMode) vm;
        }
        CodeGraphPopupMode popupMode = new CodeGraphPopupMode(codeGraph, defaultPopupMode);
        editMode.setPopupMode(popupMode);

        editMode.allowMoveSelection(true);
        editMode.allowMoving(true);
        editMode.allowNodeCreation(false);
        editMode.allowEdgeCreation(false);
    }

    @Override
    public String getNodeTooltip(CodeGraphNode codeGraphNode) {
        CodeGraphNode.Type type = codeGraphNode.getType();
        switch (type) {
            case CLASS:
                return getClassTooltip((PsiClass) codeGraphNode.getElement());
            case FIELD:
                return getFieldTooltip((PsiField) codeGraphNode.getElement());
            case INTERFACE:
                break;
            case METHOD:
                return getMethodTooltip((PsiMethod) codeGraphNode.getElement());
        }
        return "";
    }

    private String getMethodTooltip(PsiMethod psiMethod) {
        return psiMethod.getText().split("\\{")[0];
    }

    private String getFieldTooltip(PsiField psiField) {
        return psiField.getText();
    }

    private String getClassTooltip(PsiClass psiClass) {
        if (psiClass instanceof PsiAnonymousClass) {
            PsiAnonymousClass anonCls = (PsiAnonymousClass) psiClass;
            String baseClassName = anonCls.getBaseClassType().getClassName();
            return "Type of " + baseClassName;
        }
        return "";
    }

    @NotNull
    public NodeRealizer getNodeRealizer(CodeGraphNode codeGraphNode) {
        return CodeGraphRealizerFactory.createDefaultNodeRealizer(codeGraphNode);
    }
}
