package net.swengineer.codegraph.graph.presentation.popupmenu;

import com.intellij.openapi.graph.base.Node;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiClass;
import net.swengineer.codegraph.finders.ClassUsageFinder;
import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphEdge;
import net.swengineer.codegraph.graph.model.CodeGraphNode;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Set;

/**
 * Menu actions for a class's popup menu
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class ClassMenuActions {

    public static void addAllActions(CodeGraph codeGraph, JPopupMenu menu, Node node) {
        menu.add(getAddContainedMethodsAction(codeGraph, node));
        menu.add(getAddContainedFieldsAction(codeGraph, node));
        menu.add(getAddInnerClassesAction(codeGraph, node));
        menu.addSeparator();
        menu.add(getAddSuperClassesAction(codeGraph, node));
        menu.add(getAddInterfacesAction(codeGraph, node));
        menu.add(getAddExtendingOrImplementingClassesAction(codeGraph, node));
        menu.add(getAddContainingClassAction(codeGraph, node));
    }

    public static Action getAddContainedMethodsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add contained methods") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getContainedMethodsUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddContainedFieldsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add contained fields") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getContainedFieldsUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddSuperClassesAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add super classes") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getSuperClassesUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddInterfacesAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add interfaces") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getInterfacesUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddExtendingOrImplementingClassesAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add exdenteding/implementing classes") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getExtendingOrImplementingClassesUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddInnerClassesAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add inner classes") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getInnerClassesUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddContainingClassAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add containing class") {
            public void actionPerformed(ActionEvent e) {
                new ClassUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder) {
                        return finder.getContainingClassUsage();
                    }
                }.run(node);
            }
        };
    }

    private abstract static class ClassUsageAdder {
        private CodeGraph codeGraph;

        public ClassUsageAdder(CodeGraph codeGraph) {
            this.codeGraph = codeGraph;
        }

        protected abstract Set<Usage> getUsages(CodeGraph codeGraph, ClassUsageFinder finder);

        public void run(Node node) {
            Project project = codeGraph.getProject();
            CodeGraphNode cgNode = codeGraph.getCodeGraphNode(node);
            PsiClass psiClass = (PsiClass) cgNode.getElement();
            ClassUsageFinder finder = ClassUsageFinder.create(psiClass, project.getProjectScope());

            Set<Usage> usages = getUsages(codeGraph, finder);
            for (Usage usage : usages) {
                codeGraph.getDataModel().addEdge(new CodeGraphEdge(usage));
            }

            codeGraph.updateGraph();
            codeGraph.fitContent();
        }
    }
}
