package net.swengineer.codegraph.graph.presentation.popupmenu;

import com.intellij.openapi.graph.base.Node;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiMethod;
import net.swengineer.codegraph.finders.MethodUsageFinder;
import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphEdge;
import net.swengineer.codegraph.graph.model.CodeGraphNode;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Set;

/**
 * Menu actions for a method's popup menu
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class MethodMenuActions {

    public static void addAllActions(CodeGraph codeGraph, JPopupMenu menu, Node node) {
        menu.add(getAddContainingClassAction(codeGraph, node));
        menu.addSeparator();
        menu.add(getAddCalledMethodAction(codeGraph, node));
        menu.add(getAddCallingMethodsAction(codeGraph, node));
        menu.addSeparator();
        menu.add(getAddSuperMethodsAction(codeGraph, node));
        menu.add(getAddOverridingMethodsAction(codeGraph, node));
        menu.addSeparator();
        menu.add(getAddUsedFieldsAction(codeGraph, node));
        menu.add(getAddContainedAnonymousClassesAction(codeGraph, node));
    }

    private static Action getAddContainedAnonymousClassesAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add contained anonymous classes") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getContainedAnonymousClassesUsages();
                    }
                }.run(node);
            }
        };
    }

    private static Action getAddUsedFieldsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add used fields") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getUsedFieldsUsages();
                    }
                }.run(node);
            }
        };
    }

    private static Action getAddSuperMethodsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add super methods") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getSuperMethodsUsages();
                    }
                }.run(node);
            }
        };
    }

    private static Action getAddContainingClassAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add containing class") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getContainingClassUsage();
                    }
                }.run(node);
            }
        };
    }

    private static Action getAddCalledMethodAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add called methods") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getCalledMethodUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddCallingMethodsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add calling methods") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getCallingMethodsUsages();
                    }
                }.run(node);
            }
        };
    }

    public static Action getAddOverridingMethodsAction(final CodeGraph codeGraph, final Node node) {
        return new AbstractAction("Add overriding methods") {
            public void actionPerformed(ActionEvent e) {
                new MethodUsageAdder(codeGraph) {
                    protected Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder) {
                        return finder.getOverridingMethodsUsages();
                    }
                }.run(node);
            }
        };
    }

    private abstract static class MethodUsageAdder {
        private CodeGraph codeGraph;

        public MethodUsageAdder(CodeGraph codeGraph) {
            this.codeGraph = codeGraph;
        }

        protected abstract Set<Usage> getUsages(CodeGraph codeGraph, MethodUsageFinder finder);

        public void run(Node node) {
            Project project = codeGraph.getProject();
            CodeGraphNode cgNode = codeGraph.getCodeGraphNode(node);
            PsiMethod psiMethod = (PsiMethod) cgNode.getElement();
            MethodUsageFinder finder = MethodUsageFinder.create(psiMethod, project.getProjectScope());

            Set<Usage> usages = getUsages(codeGraph, finder);
            for (Usage usage : usages) {
                codeGraph.getDataModel().addEdge(new CodeGraphEdge(usage));
            }

            codeGraph.updateGraph();
            codeGraph.fitContent();
        }
    }
}
