package net.swengineer.codegraph.plugin.actions;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiField;
import com.intellij.psi.PsiManager;
import com.intellij.psi.PsiMethod;
import com.intellij.psi.search.PsiElementProcessor;
import net.swengineer.codegraph.finders.ClassUsageFinder;
import net.swengineer.codegraph.finders.FieldUsageFinder;
import net.swengineer.codegraph.finders.MethodUsageFinder;
import net.swengineer.codegraph.finders.Usage;
import net.swengineer.codegraph.graph.CodeGraph;
import net.swengineer.codegraph.graph.model.CodeGraphEdge;
import net.swengineer.codegraph.plugin.project.CodeGraphToolWindow;

import java.util.Set;

/**
 * An action for adding the entire project to the Code Graph
 *
 * @author Aviram Segal
 * @version 1.0
 */
public class AddEntireProjectAction extends AnAction {
    private Project project;

    public void actionPerformed(final AnActionEvent e) {
        project = e.getData(DataKeys.PROJECT);
        ProgressManager.getInstance().runProcessWithProgressSynchronously(
                new Runnable() {
                    public void run() {
                        doAction();
                    }
                },
                "Add Entire Project",
                false,
                project);
    }

    private void doAction() {

        assert project != null;
        PsiManager manager;
        manager = PsiManager.getInstance(project);


        final CodeGraph codeGraph = project.getUserData(CodeGraphToolWindow.CODE_GRAPH_KEY);
        codeGraph.clear();
        addAllUsages(codeGraph, project, manager);
        codeGraph.updateGraph();
        codeGraph.fitContent();
    }

    private void addAllUsages(final CodeGraph codeGraph, Project project, PsiManager manager) {
        PsiElementProcessor<PsiClass> processor = new PsiElementProcessor<PsiClass>() {
            public boolean execute(PsiClass element) {
                addAllClassUsages(codeGraph, element);
                PsiMethod[] methods = element.getMethods();
                for (PsiMethod psiMethod : methods) {
                    addMethodUsages(codeGraph, psiMethod);
                }
                PsiField[] fields = element.getFields();
                for (PsiField psiField : fields) {
                    addFieldUsages(codeGraph, psiField);
                }
                return true;
            }
        };
        manager.getSearchHelper().processAllClasses(processor, project.getProjectScope());
    }

    private void addAllClassUsages(CodeGraph codeGraph, PsiClass psiClass) {
        Set<Usage> usages = ClassUsageFinder.create(psiClass, project.getProjectScope()).getAllUsages();
        addAllUsages(codeGraph, usages);
    }

    private void addMethodUsages(CodeGraph codeGraph, PsiMethod psiMethod) {
        Set<Usage> usages = MethodUsageFinder.create(psiMethod, project.getProjectScope()).getAllUsages();
        addAllUsages(codeGraph, usages);
    }

    private void addFieldUsages(CodeGraph codeGraph, PsiField psiField) {
        Set<Usage> usages = FieldUsageFinder.create(psiField, project.getProjectScope()).getAllUsages();
        addAllUsages(codeGraph, usages);
    }

    private void addAllUsages(CodeGraph codeGraph, Set<Usage> usages) {
        for (Usage usage : usages) {
            CodeGraphEdge edge = new CodeGraphEdge(usage);
            codeGraph.getDataModel().addEdge(edge);
        }
    }
}
